import logging
import typing
import uuid
from os.path import join

import requests

from .utils import to_json

ArtifactId = typing.Union[int, str]


class ApiClient:
    """
    Client that exposes required source aggregation service endpoints

    Adds logging and tracing of requests and responses in debug mode. Raises on errors during request.
    """

    def __init__(self, endpoint, token):
        self._endpoint = endpoint
        self._token = token
        self.artifacts = Artifact(self)

    def exec_request(self, method, url, **kwargs):
        """ Executes a request, assigning a unique id beforehand and throwing on 4xx / 5xx """
        reqid = str(uuid.uuid4())

        logging.debug(
            f"{self.__class__.__qualname__} -> {method.upper()} {url} {reqid=}"
        )

        # requests.post / requests.get / ...
        method_exec = getattr(requests, method.lower())

        headers = self._build_headers()
        response = method_exec(url, headers=headers, **kwargs)

        status_code = response.status_code
        content_length = len(response.content or "")
        logging.debug(
            f"{self.__class__.__qualname__} <- {status_code} {content_length} {reqid=}"
        )

        # raise by default to halt further exec and bubble
        response.raise_for_status()

        return to_json(response)

    def build_url(self, *paths):
        return join(self._endpoint, *paths)

    def _build_headers(self):
        return {
            "Accept": "application/json",
            "User-Agent": "SAS-Python/0.0.1",
            "Authorization": f"Bearer {self._token}",
        }


class Artifact:
    def __init__(self, client):
        self.client = client

    def list(self, params: dict = None):
        response = self.client.exec_request(
            method="GET",
            url=self.client.build_url("artifact"),
            params=(params or {}),
        )
        return response or []

    def get(self, artifact_id: str):
        response = self.client.exec_request(
            method="GET",
            url=self.client.build_url(f"artifact/{artifact_id}"),
        )
        return response or {}

    def export(self, artifacts):
        ids = ",".join(str(artifact.get("id")) for artifact in artifacts)
        response = self.client.exec_request(
            method="POST",
            url=self.client.build_url(f"artifact/{ids}/export"),
        )
        return response.get("id") or []

    def ignore(self, artifacts):
        ids = ",".join(str(artifact.get("id")) for artifact in artifacts)
        response = self.client.exec_request(
            method="POST",
            url=self.client.build_url(f"artifact/{ids}/ignore"),
        )
        return response.get("id") or []



import logging


def to_json(response):
    res_decoded, decoding_err = try_json(response)

    if decoding_err:
        # decoding error, report, but assume unrecoverable
        logging.exception(decoding_err)
        return None

    if not isinstance(res_decoded, dict):
        logging.error(f"Invalid response type: {type(res_decoded)} (expected: dict)")
        return None

    return res_decoded


def try_json(response):
    """ Decodes response as json, returns decoded and optional error """
    try:
        return response.json(), None
    except ValueError as e:
        return None, e



from setuptools import setup

requires = [
    "requests>=2.25.1",
]
test_requirements = []

about = {
    "name": "source_aggregation",
    "description": "An API wrapper for the Source Aggregation Service.",
    "url": "https://github.com/wepublic-nl",
    "version": "0.1.2",
    "author": "Wepublic",
    "email": "stakeholderintel@wepublic.nl",
    "license": "Other/Proprietary License",
    "copyright": "All rights reserved",
}

setup(
    name=about["name"],
    version=about["version"],
    description=about["description"],
    author=about["author"],
    author_email=about["email"],
    url=about["url"],
    packages=["source_aggregation"],
    package_dir={"source_aggregation": "source_aggregation"},
    include_package_data=True,
    python_requires="!=3.0.*, !=3.1.*, !=3.2.*, !=3.3.*, !=3.4.*, !=3.5.*, !=3.6.*, !=3.7.*",
    install_requires=requires,
    license=about["license"],
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Intended Audience :: Developers",
        "Natural Language :: English",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: Implementation :: CPython",
        "Programming Language :: Python :: Implementation :: PyPy",
    ],
    project_urls={
        "Source": "https://github.com/wepublic-nl",
    },
)
